@@ -637,11 +637,11 @@ static struct grub_disk_dev grub_biosdisk_dev =
   {
     .name = "biosdisk",
     .id = GRUB_DISK_DEVICE_BIOSDISK_ID,
-    .iterate = grub_biosdisk_iterate,
-    .open = grub_biosdisk_open,
-    .close = grub_biosdisk_close,
-    .read = grub_biosdisk_read,
-    .write = grub_biosdisk_write,
+    .disk_iterate = grub_biosdisk_iterate,
+    .disk_open = grub_biosdisk_open,
+    .disk_close = grub_biosdisk_close,
+    .disk_read = grub_biosdisk_read,
+    .disk_write = grub_biosdisk_write,
     .next = 0
   };
 
