@@ -292,10 +292,6 @@ void
 grub_disk_close (grub_disk_t disk)
 {
   grub_partition_t part;
-
-  if (disk == NULL)
-    return;
-
   grub_dprintf ("disk", "Closing `%s'.\n", disk->name);
 
   if (disk->dev && disk->dev->disk_close)
@@ -345,7 +341,7 @@ grub_disk_read_small_real (grub_disk_t disk, grub_disk_addr_t sector,
       < (disk->total_sectors << (disk->log_sector_size - GRUB_DISK_SECTOR_BITS)))
     {
       grub_err_t err;
-      err = (disk->dev->disk_read) (disk, grub_disk_to_native_sector (disk, sector),
+      err = (disk->dev->disk_read) (disk, transform_sector (disk, sector),
 				    1U << (GRUB_DISK_CACHE_BITS
 					   + GRUB_DISK_SECTOR_BITS
 					   - disk->log_sector_size), tmp_buf);
@@ -380,8 +376,8 @@ grub_disk_read_small_real (grub_disk_t disk, grub_disk_addr_t sector,
     tmp_buf = grub_malloc (num << disk->log_sector_size);
     if (!tmp_buf)
       return grub_errno;
-
-    if ((disk->dev->disk_read) (disk, grub_disk_to_native_sector (disk, aligned_sector),
+    
+    if ((disk->dev->disk_read) (disk, transform_sector (disk, aligned_sector),
 				num, tmp_buf))
       {
 	grub_error_push ();
@@ -406,10 +402,10 @@ grub_disk_read_small (grub_disk_t disk, grub_disk_addr_t sector,
   if (err)
     return err;
   if (disk->read_hook)
-    err = (disk->read_hook) (sector + (offset >> GRUB_DISK_SECTOR_BITS),
-			     offset & (GRUB_DISK_SECTOR_SIZE - 1),
-			     size, buf, disk->read_hook_data);
-  return err;
+    (disk->read_hook) (sector + (offset >> GRUB_DISK_SECTOR_BITS),
+		       offset & (GRUB_DISK_SECTOR_SIZE - 1),
+		       size, disk->read_hook_data);
+  return GRUB_ERR_NONE;
 }
 
 /* Read data from the disk.  */
@@ -487,14 +483,14 @@ grub_disk_read (grub_disk_t disk, grub_disk_addr_t sector,
 	{
 	  grub_disk_addr_t i;
 
-	  err = (disk->dev->disk_read) (disk, grub_disk_to_native_sector (disk, sector),
+	  err = (disk->dev->disk_read) (disk, transform_sector (disk, sector),
 					agglomerate << (GRUB_DISK_CACHE_BITS
 							+ GRUB_DISK_SECTOR_BITS
 							- disk->log_sector_size),
 					buf);
 	  if (err)
 	    return err;
-
+	  
 	  for (i = 0; i < agglomerate; i ++)
 	    grub_disk_cache_store (disk->dev->id, disk->id,
 				   sector + (i << GRUB_DISK_CACHE_BITS),
@@ -505,11 +501,11 @@ grub_disk_read (grub_disk_t disk, grub_disk_addr_t sector,
 
 	  if (disk->read_hook)
 	    (disk->read_hook) (sector, 0, agglomerate << (GRUB_DISK_CACHE_BITS + GRUB_DISK_SECTOR_BITS),
-			       buf, disk->read_hook_data);
+			       disk->read_hook_data);
 
 	  sector += agglomerate << GRUB_DISK_CACHE_BITS;
 	  size -= agglomerate << (GRUB_DISK_CACHE_BITS + GRUB_DISK_SECTOR_BITS);
-	  buf = (char *) buf
+	  buf = (char *) buf 
 	    + (agglomerate << (GRUB_DISK_CACHE_BITS + GRUB_DISK_SECTOR_BITS));
 	}
 
@@ -517,7 +513,7 @@ grub_disk_read (grub_disk_t disk, grub_disk_addr_t sector,
 	{
 	  if (disk->read_hook)
 	    (disk->read_hook) (sector, 0, (GRUB_DISK_CACHE_SIZE << GRUB_DISK_SECTOR_BITS),
-			       buf, disk->read_hook_data);
+			       disk->read_hook_data);
 	  sector += GRUB_DISK_CACHE_SIZE;
 	  buf = (char *) buf + (GRUB_DISK_CACHE_SIZE << GRUB_DISK_SECTOR_BITS);
 	  size -= (GRUB_DISK_CACHE_SIZE << GRUB_DISK_SECTOR_BITS);
@@ -537,7 +533,7 @@ grub_disk_read (grub_disk_t disk, grub_disk_addr_t sector,
 }
 
 grub_uint64_t
-grub_disk_native_sectors (grub_disk_t disk)
+grub_disk_get_size (grub_disk_t disk)
 {
   if (disk->partition)
     return grub_partition_get_len (disk->partition);
